//	By default, p_DisplayRooms() procs do absolutely nothing.
//	You will have to define them for your interface.
p_RoomHandler/Demo

	p_DisplayRoomsToViewer(Viewer,AutomatedCall=FALSE)
		//	There's no sense preparing interfaces for
		//	those who can't use them.
		if ( ismob(Viewer) )
			var/mob/M		= Viewer
			if (!M.client)	return

		//	How you define this depends on your interface.
		//	I'm going to assume that we're displaying
		//	to everyone when Viewer is null.
		//	You don't have to.
		var/rooms	= src.p_GetRooms()
		winset(Viewer,"rooms","cells=1x[length(rooms) + length(src.p_GetPeople())]")

		//	I'm going to use topic links for joining rooms.
		//	Players will be displayed underneath each link.
		//	Again, you don't have to.
		//	This just fits the interface used in this demo.
		var
			p_Room/R
			RNumber	= 1
			row		= 1
			person
		for(R in rooms)
			Viewer	<< output("<a href='?src=\ref[R]'>Join Room #[RNumber]</a>","rooms:1,[row++]")
			//	Note the p_ proc used here.
			//	p_Rooms are p_PeopleHandlers.
			//	A full list of their procs can be found in
			//	the readMe file of my People library.
			for(person in R.p_GetListOfAll())
				Viewer	<< output("[person]","rooms:1,[row++]")
			RNumber++

	//	The same notes of the above proc apply to this as well.
	//	The only difference is that this proc is called
	//	automatically and displays to everyone.
	p_DisplayRoomsToAll(AutomatedCall=FALSE)
		//	You might notice me passing AutomatedCall around.
		//
		//	Sometimes you make a bunch of changes at once.
		//	It is therefore beneficial to avoid updating
		//	the display for every change.  You can call it
		//	just once later with a FALSE Automated call.
		if (AutomatedCall)	return
		var
			list
				rooms	= src.p_GetRooms()
				people	= src.p_GetPeople()
			p_Room/R
			RNumber	= 1
			row		= 1
			person

		src.p_WinsetAll("rooms","cells=1x[length(rooms) + length(people)]")

		for(R in rooms)
			people	<< output("<a href='?src=\ref[R]'>Join Room #[RNumber]</a>","rooms:1,[row++]")
			for(person in R.p_GetListOfAll())
				people	<< output("[person]","rooms:1,[row++]")
			RNumber++


//	We need to create a Room Handler somewhere.
var/p_RoomHandler/Demo/gRooms	= new


//	As long as I'm using Topic() links, let's do that here.
p_Room/Demo
	Topic(href,href_list)
		//	All that really matters is that you have some
		//	method of moving between rooms in your interface.
		gRooms.p_Move(usr,src,FALSE)

	//	Let's update the room/people list display when players
	//	move between rooms.  If you're not displaying people
	//	at the same time, then you probably don't want to
	//	bother.
	p_Entered(Person,p_Room/OldRoom,AutomatedCall=FALSE)
		..(Person,OldRoom,AutomatedCall)
		gRooms.p_DisplayRoomsToAll(AutomatedCall)

//	Let's create some rooms to jump around.
//	I'm going to just use 10 for the demo.
//	However, you're not stuck with a static number.
world
	New()
		for(var/index = 1 to 10)
			gRooms.p_AddRoom(new/p_Room/Demo)


mob
	Login()
		..()
		//	Disconnected clients can have their
		//	interfaces reset automatically.
		gRooms.p_Reconnected(src)

		//	We might also want to add a person to
		//	a default room so they can communicate.
		//	You can make this a special lobby room.
		//	However, I'm just going to pick the
		//	first room we created.
		var/p_Room/R	= gRooms.p_Find(src)
		if (!R)
			var/list/rooms	= gRooms.p_GetRooms()
			gRooms.p_Move(src,rooms[1],FALSE)

	//	Let's add some communication verbs.
	verb
		//	We can speak in a single room...
		say(Blah as text)
			if (!Blah)	return
			var/p_Room/R	= gRooms.p_Find(src)
			if (R)	R.p_OutputAll("<strong>[src]:</strong> [Blah]","chat")

		//	...or we can speak across all rooms.
		worldsay(Blah as text)
			if (!Blah)	return
			gRooms.p_OutputAll("<span class='global'>Global> <strong>[src]:</strong> [Blah]</span>","chat")
