p_RoomHandler

	//==========================================================================
	var/list/p_rooms

	//==========================================================================
	New()
		..()
		src.p_rooms	= list()

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Adds Room to the handler.
		p_AddRoom(p_Room/Room,AutomatedCall=FALSE)
			if ( istype(Room) && !(Room in src.p_rooms) )
				src.p_rooms	+= Room
				src.p_DisplayRoomsToAll(AutomatedCall)
				return TRUE
			return FALSE

		//----------------------------------------------------------------------
		//	Returns the p_Room that holds Person or null if it doesn't exist.
		p_Find(Person)
			if (!Person)	return null
			for(var/p_Room/R in src.p_rooms)
				if ( R.p_IsListed(Person) )
					return R
			return null

		//----------------------------------------------------------------------
		//	Returns a copy of the rooms list.
		p_GetRooms()
			return src.p_rooms.Copy()

		//----------------------------------------------------------------------
		//	Returns a list of all players, waiters and spectators in all rooms.
		p_GetPeople()
			.										= list()
			for(var/p_Room/R in src.p_rooms)	.	+= R.p_GetListOfAll()

		//----------------------------------------------------------------------
		//	Returns a list of all players in all rooms.
		p_GetPlayers()
			.										= list()
			for(var/p_Room/R in src.p_rooms)	.	+= R.p_GetListOfPlayers()

		//----------------------------------------------------------------------
		//	Returns a list of all spectators in all rooms.
		p_GetSpectators()
			.										= list()
			for(var/p_Room/R in src.p_rooms)	.	+= R.p_GetListOfSpectators()

		//----------------------------------------------------------------------
		//	Returns a list of all waiters in all rooms.
		p_GetWaiters()
			.										= list()
			for(var/p_Room/R in src.p_rooms)	.	+= R.p_GetListOfWaiters()

		//----------------------------------------------------------------------
		//	Move Person to Room.  Returns TRUE on success.
		p_Move(Person,p_Room/Room,AutomatedCall=FALSE)
			if ( !(Room in src.p_rooms) || !Room.p_Enter(Person) )
				return FALSE

			var/p_Room/oldRoom					= src.p_Find(Person)
			if (oldRoom)
				if ( !oldRoom.p_Exit(Person) )	return FALSE
				oldRoom.p_Exited(Person,AutomatedCall)

			Room.p_Entered(Person,oldRoom,AutomatedCall)

			return TRUE

		//----------------------------------------------------------------------
		//	Removes Room to the handler.
		p_RemoveRoom(p_Room/Room,AutomatedCall=FALSE)
			.		= src.p_rooms.Remove(Room)
			if (.)	src.p_DisplayRoomsToAll(AutomatedCall)


		////////////////////////////////////////////////////////////////////////
		//
		//	Interface
		//
		////////////////////////////////////////////////////////////////////////

		//----------------------------------------------------------------------
		//	This can be defined to add a custom interface.
		//	Called automatically by p_Reconnected().
		//	Returns TRUE if Person can use an interface.
		p_AddInterface(Person)
			var/mob/M	= Person
			return (istype(M) && M.client)

		//----------------------------------------------------------------------
		//	This does pretty much nothing by default.
		//	Calling it when mobs log out could help with library extensions.
		p_Disconnected(Person)
			var/p_Room/R	= src.p_Find(Person)
			if (R)			R.p_Disconnected(Person)

		//----------------------------------------------------------------------
		//	In theory, this should display any room stats to everyone.
		//	This is called automatically by p_Move().
		//	By default, this does absolutely nothing.
		p_DisplayRoomsToAll(AutomatedCall=FALSE)

		//----------------------------------------------------------------------
		//	In theory, this should display any room stats to Viewer.
		//	By default, this does absolutely nothing.
		p_DisplayRoomsToViewer(Viewer)

		//----------------------------------------------------------------------
		//	Outputs to all people in all rooms.
		p_OutputAll(Message,Control)
			for(var/p_Room/R in src.p_rooms)
				R.p_OutputAll(Message,Control)

		//----------------------------------------------------------------------
		//	Outputs to all players in all rooms.
		p_OutputPlayers(Message,Control)
			for(var/p_Room/R in src.p_rooms)
				R.p_OutputPlayers(Message,Control)

		//----------------------------------------------------------------------
		//	Outputs to all spectators in all rooms.
		p_OutputSpectators(Message,Control)
			for(var/p_Room/R in src.p_rooms)
				R.p_OutputSpectators(Message,Control)

		//----------------------------------------------------------------------
		//	Resets interfaces for reconnecting players.
		p_Reconnected(Person)
			src.p_AddInterface(Person)
			var/p_Room/R	= src.p_Find(Person)
			if (R)			R.p_Reconnected(Person)

		//----------------------------------------------------------------------
		//	This can be defined to remove a custom interface.
		//	Returns TRUE if Person can use an interface.
		p_RemoveInterface(Person)
			var/mob/M	= Person
			return (istype(M) && M.client)

		//----------------------------------------------------------------------
		//	Outputs to all waiters in all rooms.
		p_OutputWaiters(Message,Control)
			for(var/p_Room/R in src.p_rooms)
				R.p_OutputWaiters(Message,Control)

		//----------------------------------------------------------------------
		//	Calls winset(Control,Params) for all people in all rooms.
		p_WinsetAll(Control,Params)
			for(var/p_Room/R in src.p_rooms)
				R.p_WinsetAll(Control,Params)

		//----------------------------------------------------------------------
		//	Calls winset(Control,Params) for all players in all rooms.
		p_WinsetPlayers(Control,Params)
			for(var/p_Room/R in src.p_rooms)
				R.p_WinsetPlayers(Control,Params)

		//----------------------------------------------------------------------
		//	Calls winset(Control,Params) for all spectators in all rooms.
		p_WinsetSpectators(Control,Params)
			for(var/p_Room/R in src.p_rooms)
				R.p_WinsetSpectators(Control,Params)

		//----------------------------------------------------------------------
		//	Calls winset(Control,Params) for all waiters in all rooms.
		p_WinsetWaiters(Control,Params)
			for(var/p_Room/R in src.p_rooms)
				R.p_WinsetWaiters(Control,Params)
